@@ -1,18 +1,12 @@
-# -*- coding: utf-8 -*-
-"""
-    tests.basic
-    ~~~~~~~~~~~~~~~~~~~~~
-
-    The basic functionality.
-
-    :copyright: 2010 Pallets
-    :license: BSD-3-Clause
-"""
+import gc
 import re
 import sys
 import time
 import uuid
+import warnings
+import weakref
 from datetime import datetime
+from platform import python_implementation
 from threading import Thread
 
 import pytest
@@ -24,7 +18,12 @@ from werkzeug.http import parse_date
 from werkzeug.routing import BuildError
 
 import flask
-from flask._compat import text_type
+
+
+require_cpython_gc = pytest.mark.skipif(
+    python_implementation() != "CPython",
+    reason="Requires CPython GC behavior",
+)
 
 
 def test_options_work(app, client):
@@ -50,6 +49,23 @@ def test_options_on_multiple_rules(app, client):
     assert sorted(rv.allow) == ["GET", "HEAD", "OPTIONS", "POST", "PUT"]
 
 
+@pytest.mark.parametrize("method", ["get", "post", "put", "delete", "patch"])
+def test_method_route(app, client, method):
+    method_route = getattr(app, method)
+    client_method = getattr(client, method)
+
+    @method_route("/")
+    def hello():
+        return "Hello"
+
+    assert client_method("/").data == b"Hello"
+
+
+def test_method_route_no_methods(app):
+    with pytest.raises(TypeError):
+        app.get("/", methods=["GET", "POST"])
+
+
 def test_provide_automatic_options_attr():
     app = flask.Flask(__name__)
 
@@ -92,12 +108,7 @@ def test_provide_automatic_options_kwarg(app, client):
     assert rv.status_code == 405
     assert sorted(rv.allow) == ["GET", "HEAD"]
 
-    # Older versions of Werkzeug.test.Client don't have an options method
-    if hasattr(client, "options"):
-        rv = client.options("/")
-    else:
-        rv = client.open("/", method="OPTIONS")
-
+    rv = client.open("/", method="OPTIONS")
     assert rv.status_code == 405
 
     rv = client.head("/")
@@ -110,11 +121,7 @@ def test_provide_automatic_options_kwarg(app, client):
     assert rv.status_code == 405
     assert sorted(rv.allow) == ["GET", "HEAD", "POST"]
 
-    if hasattr(client, "options"):
-        rv = client.options("/more")
-    else:
-        rv = client.open("/more", method="OPTIONS")
-
+    rv = client.open("/more", method="OPTIONS")
     assert rv.status_code == 405
 
 
@@ -288,7 +295,7 @@ def test_session_using_server_name_port_and_path(app, client):
 
 
 def test_session_using_application_root(app, client):
-    class PrefixPathMiddleware(object):
+    class PrefixPathMiddleware:
         def __init__(self, app, prefix):
             self.app = app
             self.prefix = prefix
@@ -333,6 +340,19 @@ def test_session_using_session_settings(app, client):
     assert "httponly" not in cookie
     assert "samesite" in cookie
 
+    @app.route("/clear")
+    def clear():
+        flask.session.pop("testing", None)
+        return "Goodbye World"
+
+    rv = client.get("/clear", "http://www.example.com:8080/test/")
+    cookie = rv.headers["set-cookie"].lower()
+    assert "session=;" in cookie
+    assert "domain=.example.com" in cookie
+    assert "path=/" in cookie
+    assert "secure" in cookie
+    assert "samesite" in cookie
+
 
 def test_session_using_samesite_attribute(app, client):
     @app.route("/")
@@ -372,7 +392,7 @@ def test_session_localhost_warning(recwarn, app, client):
     rv = client.get("/", "http://localhost:5000/")
     assert "domain" not in rv.headers["set-cookie"].lower()
     w = recwarn.pop(UserWarning)
-    assert '"localhost" is not a valid cookie domain' in str(w.message)
+    assert "'localhost' is not a valid cookie domain" in str(w.message)
 
 
 def test_session_ip_warning(recwarn, app, client):
@@ -413,7 +433,7 @@ def test_session_expiration(app, client):
 
     @app.route("/test")
     def test():
-        return text_type(flask.session.permanent)
+        return str(flask.session.permanent)
 
     rv = client.get("/")
     assert "set-cookie" in rv.headers
@@ -593,18 +613,18 @@ def test_extended_flashing(app):
 
     @app.route("/")
     def index():
-        flask.flash(u"Hello World")
-        flask.flash(u"Hello World", "error")
-        flask.flash(flask.Markup(u"<em>Testing</em>"), "warning")
+        flask.flash("Hello World")
+        flask.flash("Hello World", "error")
+        flask.flash(flask.Markup("<em>Testing</em>"), "warning")
         return ""
 
     @app.route("/test/")
     def test():
         messages = flask.get_flashed_messages()
         assert list(messages) == [
-            u"Hello World",
-            u"Hello World",
-            flask.Markup(u"<em>Testing</em>"),
+            "Hello World",
+            "Hello World",
+            flask.Markup("<em>Testing</em>"),
         ]
         return ""
 
@@ -613,9 +633,9 @@ def test_extended_flashing(app):
         messages = flask.get_flashed_messages(with_categories=True)
         assert len(messages) == 3
         assert list(messages) == [
-            ("message", u"Hello World"),
-            ("error", u"Hello World"),
-            ("warning", flask.Markup(u"<em>Testing</em>")),
+            ("message", "Hello World"),
+            ("error", "Hello World"),
+            ("warning", flask.Markup("<em>Testing</em>")),
         ]
         return ""
 
@@ -624,7 +644,7 @@ def test_extended_flashing(app):
         messages = flask.get_flashed_messages(
             category_filter=["message"], with_categories=True
         )
-        assert list(messages) == [("message", u"Hello World")]
+        assert list(messages) == [("message", "Hello World")]
         return ""
 
     @app.route("/test_filters/")
@@ -633,8 +653,8 @@ def test_extended_flashing(app):
             category_filter=["message", "warning"], with_categories=True
         )
         assert list(messages) == [
-            ("message", u"Hello World"),
-            ("warning", flask.Markup(u"<em>Testing</em>")),
+            ("message", "Hello World"),
+            ("warning", flask.Markup("<em>Testing</em>")),
         ]
         return ""
 
@@ -642,8 +662,8 @@ def test_extended_flashing(app):
     def test_filters2():
         messages = flask.get_flashed_messages(category_filter=["message", "warning"])
         assert len(messages) == 2
-        assert messages[0] == u"Hello World"
-        assert messages[1] == flask.Markup(u"<em>Testing</em>")
+        assert messages[0] == "Hello World"
+        assert messages[1] == flask.Markup("<em>Testing</em>")
         return ""
 
     # Create new test client on each test to clean flashed messages.
@@ -1106,17 +1126,17 @@ def test_enctype_debug_helper(app, client):
         with pytest.raises(DebugFilesKeyError) as e:
             client.post("/fail", data={"foo": "index.txt"})
         assert "no file contents were transmitted" in str(e.value)
-        assert 'This was submitted: "index.txt"' in str(e.value)
+        assert "This was submitted: 'index.txt'" in str(e.value)
 
 
 def test_response_types(app, client):
     @app.route("/text")
     def from_text():
-        return u"Hällo Wörld"
+        return "Hällo Wörld"
 
     @app.route("/bytes")
     def from_bytes():
-        return u"Hällo Wörld".encode("utf-8")
+        return "Hällo Wörld".encode()
 
     @app.route("/full_tuple")
     def from_full_tuple():
@@ -1137,8 +1157,10 @@ def test_response_types(app, client):
     @app.route("/response_headers")
     def from_response_headers():
         return (
-            flask.Response("Hello world", 404, {"X-Foo": "Baz"}),
-            {"X-Foo": "Bar", "X-Bar": "Foo"},
+            flask.Response(
+                "Hello world", 404, {"Content-Type": "text/html", "X-Foo": "Baz"}
+            ),
+            {"Content-Type": "text/plain", "X-Foo": "Bar", "X-Bar": "Foo"},
         )
 
     @app.route("/response_status")
@@ -1153,8 +1175,8 @@ def test_response_types(app, client):
     def from_dict():
         return {"foo": "bar"}, 201
 
-    assert client.get("/text").data == u"Hällo Wörld".encode("utf-8")
-    assert client.get("/bytes").data == u"Hällo Wörld".encode("utf-8")
+    assert client.get("/text").data == "Hällo Wörld".encode()
+    assert client.get("/bytes").data == "Hällo Wörld".encode()
 
     rv = client.get("/full_tuple")
     assert rv.data == b"Meh"
@@ -1175,7 +1197,8 @@ def test_response_types(app, client):
 
     rv = client.get("/response_headers")
     assert rv.data == b"Hello world"
-    assert rv.headers.getlist("X-Foo") == ["Baz", "Bar"]
+    assert rv.content_type == "text/plain"
+    assert rv.headers.getlist("X-Foo") == ["Bar"]
     assert rv.headers["X-Bar"] == "Foo"
     assert rv.status_code == 404
 
@@ -1221,6 +1244,7 @@ def test_response_type_errors():
     with pytest.raises(TypeError) as e:
         c.get("/none")
         assert "returned None" in str(e.value)
+        assert "from_none" in str(e.value)
 
     with pytest.raises(TypeError) as e:
         c.get("/small_tuple")
@@ -1504,8 +1528,11 @@ def test_server_name_subdomain():
     rv = client.get("/", "https://dev.local")
     assert rv.data == b"default"
 
-    # suppress Werkzeug 1.0 warning about name mismatch
-    with pytest.warns(None):
+    # suppress Werkzeug 0.15 warning about name mismatch
+    with warnings.catch_warnings():
+        warnings.filterwarnings(
+            "ignore", "Current server name", UserWarning, "flask.app"
+        )
         rv = client.get("/", "http://foo.localhost")
         assert rv.status_code == 404
 
@@ -1513,6 +1540,8 @@ def test_server_name_subdomain():
     assert rv.data == b"subdomain"
 
 
+@pytest.mark.filterwarnings("ignore::pytest.PytestUnraisableExceptionWarning")
+@pytest.mark.filterwarnings("ignore::pytest.PytestUnhandledThreadExceptionWarning")
 def test_exception_propagation(app, client):
     def apprunner(config_key):
         @app.route("/")
@@ -1605,7 +1634,7 @@ def test_url_processors(app, client):
 
 
 def test_inject_blueprint_url_defaults(app):
-    bp = flask.Blueprint("foo.bar.baz", __name__, template_folder="template")
+    bp = flask.Blueprint("foo", __name__, template_folder="template")
 
     @bp.url_defaults
     def bp_defaults(endpoint, values):
@@ -1618,22 +1647,22 @@ def test_inject_blueprint_url_defaults(app):
     app.register_blueprint(bp)
 
     values = dict()
-    app.inject_url_defaults("foo.bar.baz.view", values)
+    app.inject_url_defaults("foo.view", values)
     expected = dict(page="login")
     assert values == expected
 
     with app.test_request_context("/somepage"):
-        url = flask.url_for("foo.bar.baz.view")
+        url = flask.url_for("foo.view")
     expected = "/login"
     assert url == expected
 
 
 def test_nonascii_pathinfo(app, client):
-    @app.route(u"/киртест")
+    @app.route("/киртест")
     def index():
         return "Hello World!"
 
-    rv = client.get(u"/киртест")
+    rv = client.get("/киртест")
     assert rv.data == b"Hello World!"
 
 
@@ -1708,7 +1737,7 @@ def test_routing_redirect_debugging(app, client):
         with pytest.raises(AssertionError) as e:
             client.post("/foo", data={})
         assert "http://localhost/foo/" in str(e.value)
-        assert ("Make sure to directly send your POST-request to this URL") in str(
+        assert "Make sure to directly send your POST-request to this URL" in str(
             e.value
         )
 
@@ -1840,7 +1869,7 @@ def test_subdomain_matching():
 
     @app.route("/", subdomain="<user>")
     def index(user):
-        return "index for %s" % user
+        return f"index for {user}"
 
     rv = client.get("/", "http://mitsuhiko.localhost.localdomain/")
     assert rv.data == b"index for mitsuhiko"
@@ -1853,7 +1882,7 @@ def test_subdomain_matching_with_ports():
 
     @app.route("/", subdomain="<user>")
     def index(user):
-        return "index for %s" % user
+        return f"index for {user}"
 
     rv = client.get("/", "http://mitsuhiko.localhost.localdomain:3000/")
     assert rv.data == b"index for mitsuhiko"
@@ -1870,7 +1899,10 @@ def test_subdomain_matching_other_name(matching):
         return "", 204
 
     # suppress Werkzeug 0.15 warning about name mismatch
-    with pytest.warns(None):
+    with warnings.catch_warnings():
+        warnings.filterwarnings(
+            "ignore", "Current server name", UserWarning, "flask.app"
+        )
         # ip address can't match name
         rv = client.get("/", "http://127.0.0.1:3000/")
         assert rv.status_code == 404 if matching else 204
@@ -1893,7 +1925,7 @@ def test_multi_route_rules(app, client):
 
 
 def test_multi_route_class_views(app, client):
-    class View(object):
+    class View:
         def __init__(self, app):
             app.add_url_rule("/", "index", self.index)
             app.add_url_rule("/<test>/", "index", self.index)
@@ -1925,12 +1957,12 @@ def test_run_server_port(monkeypatch, app):
 
     # Mocks werkzeug.serving.run_simple method
     def run_simple_mock(hostname, port, application, *args, **kwargs):
-        rv["result"] = "running on %s:%s ..." % (hostname, port)
+        rv["result"] = f"running on {hostname}:{port} ..."
 
     monkeypatch.setattr(werkzeug.serving, "run_simple", run_simple_mock)
     hostname, port = "localhost", 8000
     app.run(hostname, port, debug=True)
-    assert rv["result"] == "running on %s:%s ..." % (hostname, port)
+    assert rv["result"] == f"running on {hostname}:{port} ..."
 
 
 @pytest.mark.parametrize(
@@ -1985,3 +2017,19 @@ def test_max_cookie_size(app, client, recwarn):
 
     client.get("/")
     assert len(recwarn) == 0
+
+
+@require_cpython_gc
+def test_app_freed_on_zero_refcount():
+    # A Flask instance should not create a reference cycle that prevents CPython
+    # from freeing it when all external references to it are released (see #3761).
+    gc.disable()
+    try:
+        app = flask.Flask(__name__)
+        assert app.view_functions["static"]
+        weak = weakref.ref(app)
+        assert weak() is not None
+        del app
+        assert weak() is None
+    finally:
+        gc.enable()
